@@ -96,6 +96,7 @@ class Rule(object):
 
 
 def is_conditional(line):
+    line = line.lstrip(b' ')
     return (
         line.startswith(b'ifeq') or
         line.startswith(b'ifneq') or
@@ -144,7 +145,7 @@ class Makefile(object):
         for line in joinedlines:
             if line.startswith(b'\t') and rule:
                 rule.append_line(line)
-            elif is_conditional(line) or line.startswith(b'#'):
+            elif is_conditional(line) or line.lstrip(b' ').startswith(b'#'):
                 if rule:
                     rule.append_line(line)
                 else:
