@@ -39,7 +39,7 @@ from lintian_brush.vcs import (
     browse_url_from_repo_url,
     plausible_url as plausible_vcs_url,
     sanitize_url as sanitize_vcs_url,
-    KNOWN_GITLAB_SITES,
+    is_gitlab_site,
     )
 from lintian_brush.watch import parse_watch_file
 from urllib.request import urlopen, Request
@@ -173,7 +173,7 @@ def guess_repo_from_url(url, net_access=False):
             return 'https://git.savannah.gnu.org/git/%s.git' % (
                 path_elements[1])
         return None
-    if parsed_url.netloc in KNOWN_GITLAB_SITES:
+    if is_gitlab_site(parsed_url.netloc, net_access):
         if parsed_url.path.strip('/').count('/') < 1:
             return None
         parts = parsed_url.path.split('/')
@@ -325,7 +325,7 @@ def guess_from_package_json(path, trust_package):
         else:
             url = package['bugs']
         if url:
-            yield UpstreamDatum('Bugs-Database', url, 'certain')
+            yield UpstreamDatum('Bug-Database', url, 'certain')
 
 
 def guess_from_package_xml(path, trust_package):
@@ -608,8 +608,19 @@ def guess_from_configure(path, trust_package=False):
                 yield UpstreamDatum(
                     'Name', value.decode(), 'certain', './configure')
             elif key == b'PACKAGE_BUGREPORT':
-                yield UpstreamDatum(
-                    'Bug-Submit', value.decode(), 'certain', './configure')
+                if value in (b'BUG-REPORT-ADDRESS', ):
+                    pass
+                elif b'@' in value and not value.endswith(b'gnu.org'):
+                    # Downgrade the trustworthiness of this field for most
+                    # upstreams if it contains an e-mail address. Most
+                    # upstreams seem to just set this to some random address,
+                    # and then forget about it.
+                    yield UpstreamDatum(
+                        'Bug-Submit', value.decode(), 'possible',
+                        './configure')
+                else:
+                    yield UpstreamDatum(
+                        'Bug-Submit', value.decode(), 'certain', './configure')
             elif key == b'PACKAGE_URL':
                 yield UpstreamDatum(
                     'Homepage', value.decode(), 'certain', './configure')
@@ -722,7 +733,7 @@ def guess_upstream_metadata_items(path, trust_package=False,
 
 def guess_upstream_metadata(
         path, trust_package=False, net_access=False,
-        consult_external_directory=False):
+        consult_external_directory=False, check=False):
     """Guess the upstream metadata dictionary.
 
     Args:
@@ -743,6 +754,9 @@ def guess_upstream_metadata(
         upstream_metadata, path, net_access=net_access,
         consult_external_directory=consult_external_directory)
 
+    if check:
+        check_upstream_metadata(upstream_metadata)
+
     fix_upstream_metadata(upstream_metadata)
 
     return {k: v.value for (k, v) in upstream_metadata.items()}
@@ -801,9 +815,10 @@ def extend_from_external_guesser(
 def extend_from_sf(upstream_metadata, sf_project):
     # The set of fields that sf can possibly provide:
     sf_fields = ['Homepage', 'Screenshots', 'Name']
+    sf_certainty = upstream_metadata['Archive'].certainty
 
     return extend_from_external_guesser(
-        upstream_metadata, upstream_metadata['Archive'].certainty, sf_fields,
+        upstream_metadata, sf_certainty, sf_fields,
         guess_from_sf(sf_project))
 
 
@@ -813,7 +828,7 @@ def extend_from_lp(upstream_metadata, minimum_certainty, package,
     lp_fields = ['Homepage', 'Repository', 'Name']
     lp_certainty = 'possible'
 
-    if minimum_certainty and minimum_certainty != 'possible':
+    if certainty_sufficient(lp_certainty, minimum_certainty):
         # Don't bother talking to launchpad if we're not
         # speculating.
         return
@@ -823,6 +838,19 @@ def extend_from_lp(upstream_metadata, minimum_certainty, package,
              package, distribution=distribution, suite=suite))
 
 
+def extend_from_aur(upstream_metadata, minimum_certainty, package):
+    # The set of fields that AUR can possibly provide:
+    aur_fields = ['Homepage', 'Repository']
+    aur_certainty = 'possible'
+
+    if certainty_sufficient(aur_certainty, minimum_certainty):
+        # Don't bother talking to AUR if we're not speculating.
+        return
+
+    extend_from_external_guesser(
+        upstream_metadata, aur_certainty, aur_fields, guess_from_aur(package))
+
+
 def extract_sf_project_name(url):
     m = re.fullmatch('https?://(.*).(sf|sourceforge).net/?', url)
     if m:
@@ -857,7 +885,7 @@ def bug_submit_url_from_bug_database_url(url):
         if len(path_elements) == 1:
             return urlunparse(
                 parsed_url._replace(path=parsed_url.path+'/+filebug'))
-    if parsed_url.netloc in KNOWN_GITLAB_SITES:
+    if is_gitlab_site(parsed_url.netloc):
         if len(path_elements) < 2:
             return None
         if path_elements[-1] != 'issues':
@@ -880,6 +908,83 @@ def verify_bug_database_url(url):
     return None
 
 
+def _extrapolate_repository_from_homepage(upstream_metadata, net_access):
+    repo = guess_repo_from_url(
+            upstream_metadata['Homepage'].value, net_access=net_access)
+    if repo:
+        return UpstreamDatum('Repository', repo, 'likely')
+
+
+def _extrapolate_repository_from_bug_db(upstream_metadata, net_access):
+    repo = guess_repo_from_url(
+        upstream_metadata['Bug-Database'].value, net_access=net_access)
+    if repo:
+        return UpstreamDatum('Repository', repo, 'likely')
+
+
+def _extrapolate_repository_browse_from_repository(
+        upstream_metadata, net_access):
+    browse_url = browse_url_from_repo_url(
+            upstream_metadata['Repository'].value)
+    if browse_url:
+        return UpstreamDatum(
+            'Repository-Browse', browse_url,
+            upstream_metadata['Repository'].certainty)
+
+
+def _extrapolate_repository_from_repository_browse(
+        upstream_metadata, net_access):
+    repo = guess_repo_from_url(
+        upstream_metadata['Repository-Browse'].value,
+        net_access=net_access)
+    if repo:
+        return UpstreamDatum(
+            'Repository', repo,
+            upstream_metadata['Repository-Browse'].certainty)
+
+
+def _extrapolate_bug_database_from_repository(
+        upstream_metadata, net_access):
+    bug_db_url = guess_bug_database_url_from_repo_url(
+        upstream_metadata['Repository'].value)
+    if bug_db_url:
+        return UpstreamDatum('Bug-Database', bug_db_url, 'likely')
+
+
+def _extrapolate_bug_submit_from_bug_db(
+        upstream_metadata, net_access):
+    bug_submit_url = bug_submit_url_from_bug_database_url(
+        upstream_metadata['Bug-Database'].value)
+    if bug_submit_url:
+        return UpstreamDatum(
+            'Bug-Submit', bug_submit_url,
+            upstream_metadata['Bug-Database'].certainty)
+
+
+def _copy_bug_db_field(upstream_metadata, net_access):
+    ret = UpstreamDatum(
+        'Bug-Database',
+        upstream_metadata['Bugs-Database'].value,
+        upstream_metadata['Bugs-Database'].certainty,
+        upstream_metadata['Bugs-Database'].origin)
+    del upstream_metadata['Bugs-Database']
+    return ret
+
+
+EXTRAPOLATE_FNS = [
+    ('Homepage', 'Repository', _extrapolate_repository_from_homepage),
+    ('Bugs-Database', 'Bug-Database', _copy_bug_db_field),
+    ('Bug-Database', 'Repository', _extrapolate_repository_from_bug_db),
+    ('Repository', 'Repository-Browse',
+     _extrapolate_repository_browse_from_repository),
+    ('Repository-Browse', 'Repository',
+     _extrapolate_repository_from_repository_browse),
+    ('Repository', 'Bug-Database',
+     _extrapolate_bug_database_from_repository),
+    ('Bug-Database', 'Bug-Submit', _extrapolate_bug_submit_from_bug_db),
+]
+
+
 def extend_upstream_metadata(upstream_metadata, path, minimum_certainty=None,
                              net_access=False,
                              consult_external_directory=False):
@@ -911,54 +1016,19 @@ def extend_upstream_metadata(upstream_metadata, path, minimum_certainty=None,
             pass
         else:
             extend_from_lp(upstream_metadata, minimum_certainty, package)
-    if ('Homepage' in upstream_metadata and
-            'Repository' not in upstream_metadata):
-        repo = guess_repo_from_url(
-                upstream_metadata['Homepage'].value, net_access=net_access)
-        if repo:
-            upstream_metadata['Repository'] = UpstreamDatum(
-                'Repository', repo, 'likely')
-    if ('Bug-Database' in upstream_metadata and
-            'Repository' not in upstream_metadata):
-        repo = guess_repo_from_url(
-            upstream_metadata['Bug-Database'].value, net_access=net_access)
-        if repo:
-            upstream_metadata['Repository'] = UpstreamDatum(
-                'Repository', repo, 'likely')
-    if ('Repository' in upstream_metadata and
-            'Repository-Browse' not in upstream_metadata):
-        browse_url = browse_url_from_repo_url(
-                upstream_metadata['Repository'].value)
-        if browse_url:
-            upstream_metadata['Repository-Browse'] = UpstreamDatum(
-                'Repository-Browse', browse_url,
-                upstream_metadata['Repository'].certainty)
-    if ('Repository-Browse' in upstream_metadata and
-            'Repository' not in upstream_metadata):
-        repo = guess_repo_from_url(
-            upstream_metadata['Repository-Browse'].value,
-            net_access=net_access)
-        if repo:
-            upstream_metadata['Repository'] = UpstreamDatum(
-                'Repository', repo,
-                upstream_metadata['Repository-Browse'].certainty)
-    if ('Repository' in upstream_metadata and
-            'Bug-Database' not in upstream_metadata):
-        bug_db_url = guess_bug_database_url_from_repo_url(
-            upstream_metadata['Repository'].value)
-        bug_db_certainty = 'likely'
-        if bug_db_url and certainty_sufficient(
-                bug_db_certainty, minimum_certainty):
-            upstream_metadata['Bug-Database'] = UpstreamDatum(
-                'Bug-Database', bug_db_url, bug_db_certainty)
-    if ('Bug-Database' in upstream_metadata and
-            'Bug-Submit' not in upstream_metadata):
-        bug_submit_url = bug_submit_url_from_bug_database_url(
-            upstream_metadata['Bug-Database'].value)
-        if bug_submit_url:
-            upstream_metadata['Bug-Submit'] = UpstreamDatum(
-                'Bug-Submit', bug_submit_url,
-                upstream_metadata['Bug-Database'].certainty)
+        extend_from_aur(upstream_metadata, minimum_certainty, package)
+    changed = True
+    while changed:
+        changed = False
+        for from_field, to_field, fn in EXTRAPOLATE_FNS:
+            if (to_field in upstream_metadata or
+                    from_field not in upstream_metadata):
+                continue
+            result = fn(upstream_metadata, net_access)
+            if result and certainty_sufficient(
+                    result.certainty, minimum_certainty):
+                upstream_metadata[to_field] = result
+                changed = True
 
 
 def probe_upstream_branch_url(url, version=None):
@@ -997,6 +1067,24 @@ def probe_upstream_branch_url(url, version=None):
         breezy.ui.ui_factory = old_ui
 
 
+def verify_screenshots(urls):
+    headers = {'User-Agent': USER_AGENT}
+    for url in urls:
+        try:
+            response = urlopen(
+                Request(url, headers=headers, method='HEAD'),
+                timeout=DEFAULT_URLLIB_TIMEOUT)
+        except urllib.error.HTTPError as e:
+            if e.status == 404:
+                yield url, False
+            else:
+                yield url, None
+        else:
+            assert response is not None
+            # TODO(jelmer): Check content-type?
+            yield url, True
+
+
 def check_upstream_metadata(upstream_metadata, version=None):
     """Check upstream metadata.
 
@@ -1018,6 +1106,19 @@ def check_upstream_metadata(upstream_metadata, version=None):
     if bug_database and bug_database.certainty == 'likely':
         if verify_bug_database_url(bug_database.value):
             bug_database.certainty = 'certain'
+    screenshots = upstream_metadata.get('Screenshots')
+    if screenshots and screenshots.certainty == 'likely':
+        newvalue = []
+        screenshots.certainty = 'certain'
+        for i, (url, status) in enumerate(verify_screenshots(
+                screenshots.value)):
+            if status is True:
+                newvalue.append(url)
+            elif status is False:
+                pass
+            else:
+                screenshots.certainty = 'likely'
+        screenshots.value = newvalue
 
 
 def parse_pkgbuild_variables(f):
@@ -1050,17 +1151,23 @@ def parse_pkgbuild_variables(f):
 
 
 def guess_from_aur(package):
-    url = (
-        'https://aur.archlinux.org/cgit/aur.git/plain/PKGBUILD?h=%s-git' %
-        package)
-    headers = {'User-Agent': USER_AGENT}
-    try:
-        f = urlopen(
-            Request(url, headers=headers),
-            timeout=DEFAULT_URLLIB_TIMEOUT)
-    except urllib.error.HTTPError as e:
-        if e.status != 404:
-            raise
+    vcses = ['git', 'bzr', 'hg']
+    for vcs in vcses:
+        url = (
+            'https://aur.archlinux.org/cgit/aur.git/plain/PKGBUILD?h=%s-%s' %
+            (package, vcs))
+        headers = {'User-Agent': USER_AGENT}
+        try:
+            f = urlopen(
+                Request(url, headers=headers),
+                timeout=DEFAULT_URLLIB_TIMEOUT)
+        except urllib.error.HTTPError as e:
+            if e.status != 404:
+                raise
+            continue
+        else:
+            break
+    else:
         return
 
     variables = parse_pkgbuild_variables(f)
@@ -1079,7 +1186,8 @@ def guess_from_aur(package):
                 unique_name, url = value.split('::', 1)
             except ValueError:
                 url = value
-            if url.startswith('git+'):
+            url = url.replace('#branch=', ',branch=')
+            if any([url.startswith(vcs+'+') for vcs in vcses]):
                 yield 'Repository', sanitize_vcs_url(url)
         if key == '_gitroot':
             yield 'Repository', sanitize_vcs_url(value[0])
@@ -1191,6 +1299,9 @@ if __name__ == '__main__':
         '--disable-net-access',
         help='Do not probe external services.',
         action='store_true', default=False)
+    parser.add_argument(
+        '--check', action='store_true',
+        help='Check guessed metadata against external sources.')
     parser.add_argument(
         '--consult-external-directory',
         action='store_true',
@@ -1199,5 +1310,7 @@ if __name__ == '__main__':
 
     metadata = guess_upstream_metadata(
         args.path, args.trust, not args.disable_net_access,
-        consult_external_directory=args.consult_external_directory)
+        consult_external_directory=args.consult_external_directory,
+        check=args.check)
+
     sys.stdout.write(ruamel.yaml.round_trip_dump(metadata))
